﻿using System.Collections.Generic;

using Raytracer;
using Raytracer.Math;
using Raytracer.Primitives;

namespace Rays {
    public interface Ast { }

    public class Text : Ast {
        public string Value { get; private set; }
        public Text(string value) { this.Value = value; }
    }

    public class Number : Ast {
        public double Value { get; private set; }
        public Number(double value) { this.Value = value; }
    }

    public class Symbol : Ast {
        public string Name { get; private set; }
        public Symbol(string name) { this.Name = name; }  
    }

    public class Pair : Ast {
        public string Name { get; private set; }
        public Ast Value { get; private set; }
        public Pair(string name, Ast value) { this.Name = name; this.Value = value; }
    }

    public class Vector : Ast {
        public IList<Ast> Components { get; private set; }
        public Vector(IEnumerable<Ast> components) { this.Components = new List<Ast>(components); }
    }

    public class List : Ast {
        public IEnumerable<Ast> Values { get; private set; }
        public List(IEnumerable<Ast> values) { this.Values = values; }
    }

    public class BuiltinMaterial : Ast {
        public Surface Material { get; private set; }
        public BuiltinMaterial(Surface s) { this.Material = s; }
    }

}