using System.Collections.Generic;
using System.Linq;

using Rays;



using System;



public class Parser {
	public const int _EOF = 0;
	public const int _ident = 1;
	public const int _double = 2;
	public const int _symbol = 3;
	public const int _string = 4;
	public const int maxT = 12;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

public Ast ResultAst { get; private set; }



	public Parser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.SemErr(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}


	bool WeakSeparator(int n, int syFol, int repFol) {
		int kind = la.kind;
		if (kind == n) {Get(); return true;}
		else if (StartOf(repFol)) {return false;}
		else {
			SynErr(n);
			while (!(set[syFol, kind] || set[repFol, kind] || set[0, kind])) {
				Get();
				kind = la.kind;
			}
			return StartOf(syFol);
		}
	}

	
	void Rays() {
		Ast current = null; var exprs = new List<Ast>(); 
		Expr(ref current);
		exprs.Add(current); 
		while (StartOf(1)) {
			Expr(ref current);
			exprs.Add(current); 
		}
		ResultAst = new List(exprs); 
	}

	void Expr(ref Ast result) {
		if (la.kind == 1) {
			Pair(ref result);
		} else if (StartOf(2)) {
			Atom(ref result);
		} else SynErr(13);
		Expect(5);
	}

	void Pair(ref Ast result) {
		string ident = ""; Ast res = null; 
		Expect(1);
		ident = t.val; 
		if (la.kind == 6) {
			Get();
			Atom(ref res);
			result = new Pair(ident, res); 
		} else if (la.kind == 10) {
			Ast resultList = null; 
			List(ref resultList);
			var cont = ((List)(resultList)).Values.ToList(); cont.Add(new Pair("Type", new Symbol(ident))); result = new List(cont); 
		} else SynErr(14);
	}

	void Atom(ref Ast result) {
		if (la.kind == 2) {
			Number(ref result);
		} else if (la.kind == 3) {
			Symbol(ref result);
		} else if (la.kind == 4) {
			String(ref result);
		} else if (la.kind == 7) {
			Vector(ref result);
		} else if (la.kind == 10) {
			List(ref result);
		} else SynErr(15);
	}

	void Number(ref Ast result) {
		Expect(2);
		result = new Number(double.Parse(t.val.Replace('.', ','))); 
	}

	void Symbol(ref Ast result) {
		Expect(3);
		result = new Symbol(t.val.Substring(1, t.val.Length - 1)); 
	}

	void String(ref Ast result) {
		Expect(4);
		result = new Text(t.val.Substring(1, t.val.Length - 2)); 
	}

	void Vector(ref Ast result) {
		Expect(7);
		Ast current = null; var exprs = new List<Ast>(); 
		Number(ref current);
		exprs.Add(current); 
		while (la.kind == 8) {
			Get();
			Number(ref current);
			exprs.Add(current); 
		}
		result = new Vector(exprs); 
		Expect(9);
	}

	void List(ref Ast result) {
		Expect(10);
		Ast current = null; var exprs = new List<Ast>(); 
		while (StartOf(1)) {
			Expr(ref current);
			exprs.Add(current); 
		}
		result = new List(exprs); 
		Expect(11);
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		Rays();

    Expect(0);
	}
	
	static readonly bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x,x,x, x,x},
		{x,T,T,T, T,x,x,T, x,x,T,x, x,x},
		{x,x,T,T, T,x,x,T, x,x,T,x, x,x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
	public System.IO.TextWriter errorStream = Console.Out;   // error messages go to this stream
  public string errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
  
	public void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "ident expected"; break;
			case 2: s = "double expected"; break;
			case 3: s = "symbol expected"; break;
			case 4: s = "string expected"; break;
			case 5: s = "\";\" expected"; break;
			case 6: s = "\":\" expected"; break;
			case 7: s = "\"(\" expected"; break;
			case 8: s = "\",\" expected"; break;
			case 9: s = "\")\" expected"; break;
			case 10: s = "\"{\" expected"; break;
			case 11: s = "\"}\" expected"; break;
			case 12: s = "??? expected"; break;
			case 13: s = "invalid Expr"; break;
			case 14: s = "invalid Pair"; break;
			case 15: s = "invalid Atom"; break;

			default: s = "error " + n; break;
		}
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void SemErr (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}
	
	public void SemErr (string s) {
		errorStream.WriteLine(s);
		count++;
	}
	
	public void Warning (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
	}
	
	public void Warning(string s) {
		errorStream.WriteLine(s);
	}
} // Errors


public class FatalError: Exception {
	public FatalError(string m): base(m) {}
}

